@@ -1,4 +1,5 @@
-from pythonforandroid.toolchain import PythonRecipe
+from pythonforandroid.recipe import PythonRecipe
+
 
 class CherryPyRecipe(PythonRecipe):
     version = '5.1.0'
@@ -7,4 +8,5 @@ class CherryPyRecipe(PythonRecipe):
     site_packages_name = 'cherrypy'
     call_hostpython_via_targetpython = False
 
+
 recipe = CherryPyRecipe()
