@@ -1,22 +1,14 @@
-from pythonforandroid.toolchain import Recipe, shprint, current_directory, ArchARM
+from pythonforandroid.toolchain import Recipe, current_directory, shprint
 from os.path import exists, join, realpath
-from os import uname
-import glob
 import sh
-import os
-import shutil
-
-# TODO
-# Recipe doesn't work with NDK r15c+ yet,
-# see: https://github.com/android-ndk/ndk/issues/477
 
 
 class FFMpegRecipe(Recipe):
-    version = '3.3.3'
+    version = '3.4.1'
     url = 'http://ffmpeg.org/releases/ffmpeg-{version}.tar.bz2'
     depends = ['sdl2']  # Need this to build correct recipe order
     opts_depends = ['openssl', 'ffpyplayer_codecs']
-    patches = ['patches/fix-libshine-configure.patch']
+    patches = ['patches/configure.patch']
 
     def should_build(self, arch):
         build_dir = self.get_build_dir(arch.arch)
@@ -40,7 +32,7 @@ class FFMpegRecipe(Recipe):
 
             if 'openssl' in self.ctx.recipe_build_order:
                 flags += [
-                    '--enable-openssl', 
+                    '--enable-openssl',
                     '--enable-nonfree',
                     '--enable-protocol=https,tls_openssl',
                 ]
@@ -48,7 +40,7 @@ class FFMpegRecipe(Recipe):
                 cflags += ['-I' + build_dir + '/include/']
                 ldflags += ['-L' + build_dir]
 
-            if 'ffpyplayer_codecs' in self.ctx.recipe_build_order:                
+            if 'ffpyplayer_codecs' in self.ctx.recipe_build_order:
                 # libx264
                 flags += ['--enable-libx264']
                 build_dir = Recipe.get_recipe('libx264', self.ctx).get_build_dir(arch.arch)
@@ -95,7 +87,7 @@ class FFMpegRecipe(Recipe):
 
             # other flags:
             flags += [
-                '--enable-filter=aresample,resample,crop,adelay,volume',
+                '--enable-filter=aresample,resample,crop,adelay,volume,scale',
                 '--enable-protocol=file,http',
                 '--enable-small',
                 '--enable-hwaccels',
@@ -107,16 +99,16 @@ class FFMpegRecipe(Recipe):
 
             # android:
             flags += [
-                '--target-os=android', 
-                '--cross-prefix=arm-linux-androideabi-', 
+                '--target-os=android',
+                '--cross-prefix=arm-linux-androideabi-',
                 '--arch=arm',
                 '--sysroot=' + self.ctx.ndk_platform,
                 '--enable-neon',
                 '--prefix={}'.format(realpath('.')),
             ]
             cflags += [
-                '-mfpu=vfpv3-d16', 
-                '-mfloat-abi=softfp', 
+                '-mfpu=vfpv3-d16',
+                '-mfloat-abi=softfp',
                 '-fPIC',
             ]
 
@@ -130,4 +122,5 @@ class FFMpegRecipe(Recipe):
             # copy libs:
             sh.cp('-a', sh.glob('./lib/lib*.so'), self.ctx.get_libs_dir(arch.arch))
 
+
 recipe = FFMpegRecipe()
