@@ -1,5 +1,5 @@
-from pythonforandroid.toolchain import BootstrapNDKRecipe, shprint, current_directory, info
-from os.path import exists, join
+from pythonforandroid.recipe import BootstrapNDKRecipe
+from pythonforandroid.toolchain import current_directory, shprint
 import sh
 
 
@@ -7,19 +7,15 @@ class GenericNDKBuildRecipe(BootstrapNDKRecipe):
     version = None
     url = None
 
-    depends = [('python2', 'python3crystax')]
+    depends = [('python2', 'python3', 'python3crystax')]
     conflicts = ['sdl2', 'pygame', 'sdl']
 
     def should_build(self, arch):
         return True
 
-    def get_recipe_env(self, arch=None):
-        env = super(GenericNDKBuildRecipe, self).get_recipe_env(arch)
-        py2 = self.get_recipe('python2', arch.ctx)
-        env['PYTHON2_NAME'] = py2.get_dir_name()
-        if 'python2' in self.ctx.recipe_build_order:
-            env['EXTRA_LDLIBS'] = ' -lpython2.7'
-
+    def get_recipe_env(self, arch=None, with_flags_in_cc=True, with_python=True):
+        env = super(GenericNDKBuildRecipe, self).get_recipe_env(
+            arch=arch, with_flags_in_cc=with_flags_in_cc, with_python=with_python)
         env['APP_ALLOW_MISSING_DEPS'] = 'true'
         return env
 
