@@ -1,10 +1,12 @@
-from pythonforandroid.toolchain import PythonRecipe
+from pythonforandroid.recipe import PythonRecipe
 
 """
 Privacy with BitTorrent and resilient to shut down
 
 http://www.tribler.org
 """
+
+
 class LibTriblerRecipe(PythonRecipe):
 
     version = 'devel'
@@ -12,7 +14,7 @@ class LibTriblerRecipe(PythonRecipe):
     url = 'git+https://github.com/Tribler/tribler.git'
 
     depends = ['apsw', 'cryptography', 'ffmpeg', 'libsodium', 'libtorrent', 'm2crypto',
-               'netifaces', 'openssl', 'pil', 'pycrypto', 'pyleveldb', 'python2', 'twisted',
+               'netifaces', 'openssl', 'pil', 'pycrypto', 'pyleveldb', 'twisted',
               ]
 
     python_depends = ['chardet', 'cherrypy', 'configobj', 'decorator', 'feedparser',
@@ -22,4 +24,4 @@ class LibTriblerRecipe(PythonRecipe):
     site_packages_name = 'Tribler'
 
 
-recipe = LibTriblerRecipe()
\ No newline at end of file
+recipe = LibTriblerRecipe()
