@@ -1,12 +1,17 @@
-from pythonforandroid.toolchain import PythonRecipe, current_directory, shprint
+from pythonforandroid.recipe import PythonRecipe
+from pythonforandroid.toolchain import current_directory, shprint
 import sh
 
 
 class Psycopg2Recipe(PythonRecipe):
+    """
+    Requires `libpq-dev` system dependency e.g. for `pg_config` binary.
+    """
     version = 'latest'
     url = 'http://initd.org/psycopg/tarballs/psycopg2-{version}.tar.gz'
-    depends = [('python2', 'python3crystax'), 'libpq']
+    depends = ['libpq']
     site_packages_name = 'psycopg2'
+    call_hostpython_via_targetpython = False
 
     def prebuild_arch(self, arch):
         libdir = self.ctx.get_libs_dir(arch.arch)
@@ -38,4 +43,5 @@ class Psycopg2Recipe(PythonRecipe):
                     '--root={}'.format(self.ctx.get_python_install_dir()),
                     '--install-lib=lib/python2.7/site-packages', _env=env)
 
+
 recipe = Psycopg2Recipe()
