@@ -1,10 +1,11 @@
 
-from pythonforandroid.toolchain import PythonRecipe
+from pythonforandroid.recipe import PythonRecipe
 
 
 class PyASN1Recipe(PythonRecipe):
     version = '0.1.8'
     url = 'https://pypi.python.org/packages/source/p/pyasn1/pyasn1-{version}.tar.gz'
-    depends = ['python2']
+    depends = []
+
 
 recipe = PyASN1Recipe()
