@@ -1,11 +1,11 @@
 
-from pythonforandroid.toolchain import PythonRecipe
+from pythonforandroid.recipe import PythonRecipe
 
 
 class PyOpenSSLRecipe(PythonRecipe):
-    version = '0.14'
+    version = '16.0.0'
     url = 'https://pypi.python.org/packages/source/p/pyOpenSSL/pyOpenSSL-{version}.tar.gz'
-    depends = ['openssl', 'python2', 'setuptools']
+    depends = ['openssl', 'setuptools']
     site_packages_name = 'OpenSSL'
 
     call_hostpython_via_targetpython = False
