@@ -1,8 +1,6 @@
 
-from pythonforandroid.toolchain import PythonRecipe, shprint, current_directory, ArchARM
-from os.path import exists, join
-import sh
-import glob
+from pythonforandroid.recipe import PythonRecipe
+
 
 class PySDL2Recipe(PythonRecipe):
     version = '0.9.3'
