@@ -1,4 +1,4 @@
-from pythonforandroid.toolchain import BootstrapNDKRecipe
+from pythonforandroid.recipe import BootstrapNDKRecipe
 from pythonforandroid.patching import is_arch
 
 
@@ -12,4 +12,5 @@ class LibSDL2Image(BootstrapNDKRecipe):
                'extra_cflags.patch',
                'fix_with_ndk_15_plus.patch']
 
+
 recipe = LibSDL2Image()
