@@ -1,8 +1,7 @@
 
-from pythonforandroid.toolchain import PythonRecipe, shprint, current_directory
-from os.path import exists, join
-import sh
-import glob
+from pythonforandroid.recipe import PythonRecipe
+from os.path import join
+
 
 class ZopeRecipe(PythonRecipe):
     name = 'zope'
@@ -10,7 +9,7 @@ class ZopeRecipe(PythonRecipe):
     url = 'http://pypi.python.org/packages/source/z/zope.interface/zope.interface-{version}.tar.gz'
 
     depends = ['python2']
-    
+
     def get_recipe_env(self, arch):
         env = super(ZopeRecipe, self).get_recipe_env(arch)
 
@@ -24,4 +23,5 @@ class ZopeRecipe(PythonRecipe):
 
         # Should do some deleting here
 
+
 recipe = ZopeRecipe()
