@@ -1,4 +1,5 @@
-from pythonforandroid.toolchain import PythonRecipe, current_directory
+from pythonforandroid.recipe import PythonRecipe
+from pythonforandroid.toolchain import current_directory
 import sh
 
 
@@ -9,7 +10,7 @@ class ZopeInterfaceRecipe(PythonRecipe):
     url = 'https://pypi.python.org/packages/source/z/zope.interface/zope.interface-{version}.tar.gz'
     site_packages_name = 'zope.interface'
 
-    depends = ['python2']
+    depends = [('python2', 'python3crystax')]
     patches = ['no_tests.patch']
 
     def prebuild_arch(self, arch):
